//---------------------------------------------------------------------------
// Technical functions
//---------------------------------------------------------------------------
unit TechnicalFunctions;

interface

uses
  IndicatorInterfaceUnit, SysUtils;

type
  TPriceType = (pt_Close, pt_Open, pt_High, pt_Low, pt_HL2, pt_HLC3, pt_HLCC4);
  TMAType = (ma_SMA, ma_EMA, ma_WMA, ma_SSMA);
  TValueType = (vt_Open, vt_High, vt_Low, vt_Close, vt_Volume);


// Get price by index and type
// index - index in bar array
// PriceType - type of price or combination
function  GetPrice(index: integer; PriceType: TPriceType): double;

// Register option "Apply to price" and fill array of its
// text values
procedure RegApplyToPriceOption(var option: integer; name: string = '');

// Register option "MA type" and fill array of its text values
procedure RegMATypeOption(var option: integer; name: string = '');

// Get linear regression channel parameters
// shift - shift value
// period - channel period
// PriceType - type of price
// StartValue - start price of center line
// EndValue - end price of center line
// Height - channel height
// top - channel top
// bottom - channel bottom
procedure LRCChannelParams(shift, period: integer; PriceType: TPriceType;
            var StartValue, EndValue, Height, Top, Bottom: double);

// Get moving average value
// index - index in bars array
// shift - shift value
// maType - moving average type (SMA, EMA, WMA)
// ApplyTo - price type
// prev - previous value for EMA
function  GetMA(index, shift, period: integer; maType: TMAType;
            ApplyTo: TPriceType; prev: double = 0): double;

// Get highest value in array
// ValueType - type of the value (vt_Open..vt_Volume)
// StartIndex - index from which we start search
// count - number of bars to search
function  GetHighestValue(ValueType: TValueType; StartIndex, count: integer): double;

// Get lovest value in array
// ValueType - type of the value (vt_Open..vt_Volume)
// StartIndex - index from which we start search
// count - number of bars to search
function  GetLowestValue(ValueType: TValueType; StartIndex, count: integer): double;


implementation

{-----Get price--------------------------------------------------------------}
function GetPrice(index: integer; PriceType: TPriceType): double;
begin
  case PriceType of
    pt_Close: result := Close(index);
    pt_Open:  result := Open(index);
    pt_High:  result := High(index);
    pt_Low:   result := Low(index);
    pt_HL2:   result := (High(index) + Low(index))/2;
    pt_HLC3:  result := (High(index) + Low(index) + Close(index))/3;
    pt_HLCC4: result := (High(index) + Low(index) + Close(index)*2)/4;
    else      result := 0;
  end;
end;

{-----Create "Apply to price" option-----------------------------------------}
procedure RegApplyToPriceOption(var option: integer; name: string = '');
begin
  if name = '' then
    name := 'Apply to price';
  RegOption(name, ot_EnumType, option);
  AddOptionValue(name, 'Close');
  AddOptionValue(name, 'Open');
  AddOptionValue(name, 'High');
  AddOptionValue(name, 'Low');
  AddOptionValue(name, '(High + Low)/2');
  AddOptionValue(name, '(High + Low + Close)/3');
  AddOptionValue(name, '(High + Low + Close + Close)/4');
  option := 0;
end;

{-----Create "MA type" option------------------------------------------------}
procedure RegMATypeOption(var option: integer; name: string = '');
begin
  if name = '' then
    name := 'MA type';
  RegOption(name, ot_EnumType, option);
  AddOptionValue(name, 'Simple (SMA)');
  AddOptionValue(name, 'Exponential (EMA)');
  AddOptionValue(name, 'Weighted (WMA)');
  AddOptionValue(name, 'Smoothed (SSMA)');
  option := 0;
end;

{-----Get channel params-----------------------------------------------------}
procedure LRCChannelParams(shift, period: integer; PriceType: TPriceType;
  var StartValue, EndValue, Height, Top, Bottom: double);
var
  i, x: integer;
  a, b, y, z, sum_x, sum_y, max, sum_xy, sum_x2: double;
begin
  // Variable initialization
  sum_x := 0;
  sum_y := 0;
  sum_xy := 0;
  sum_x2 := 0;

  // Calculating sums for regression line
  i := shift;
  for x:=0 to Period - 1 do
    begin
      y := GetPrice(i, PriceType);
      sum_x := sum_x + x;
      sum_y := sum_y + y;
      sum_xy := sum_xy + x*y;
      sum_x2 := sum_x2 + x*x;
      inc(i);
    end;

  // Calculating regression line
  b := (Period*sum_xy - sum_x*sum_y)/(Period*sum_x2 - sum_x*sum_x);
  a := (sum_y - b*sum_x)/Period;

  // Calculating channel height
  i := shift;
  max := 0;
  for x:=0 to Period-1 do
    begin
      y := a + b*x;
      z := abs(GetPrice(i, PriceType) - y);
      if (z > max) then max := z;
      inc(i);
    end;

  // Returning channel values
  StartValue := a + b*Period;
  EndValue := a;
  Height := max;
  Top := a + max;
  Bottom := a - max;
end;

{-----Get moving average-----------------------------------------------------}
function  GetMA(index, shift, period: integer; maType: TMAType;
  ApplyTo: TPriceType; prev: double = 0): double;
var
  i: integer;
  k, sum, weight: double;
begin
  case MAtype of
    ma_SMA:
       begin
         // SMA
         if (index + shift + period >= Bars) or (index + shift < 0) then
           result := 0
         else
           begin
             sum := 0;
             for i:=index to index + period - 1 do
               sum := sum + GetPrice(i + Shift, ApplyTo);
             result := sum/period;
           end;
       end;

    ma_EMA:
       begin
         // EMA
         k := 2/(period + 1);
         i := index + shift;
         if (i > Bars - 1) or (i < 0) then
           result := 0
         else
           begin
             if prev = 0 then
               result := GetPrice(i, ApplyTo)
             else
               result := prev + k*(GetPrice(i, ApplyTo) - prev);
           end;
       end;

    ma_WMA:
       begin
         // WMA
         if (index + shift + period >= Bars) or (index + shift < 0) then
           result := 0
         else
           begin
             sum := 0;
             weight := 0;
             for i:=0 to period - 1 do
               begin
                 sum := sum + GetPrice(index + i + shift, ApplyTo)*(period - i);
                 weight := weight + (period - i);
               end;
             result := sum/weight;
           end;
       end;

    else
      begin
        // SSMA
        if (index + shift + period >= Bars) or (index + shift < 0) then
           result := 0
         else
           begin
             if prev = 0 then
               result := GetMA(index, shift, period, ma_SMA, ApplyTo, prev)
             else
               result := (prev*period - prev + GetPrice(index + shift, ApplyTo))/period;
           end;
      end;
  end; {case}
end;

{-----Get highest value------------------------------------------------------}
function  GetHighestValue(ValueType: TValueType; StartIndex, count: integer): double;
var
  i: integer;

  function value(index: integer): double;
  begin
    case ValueType of
      vt_Open:  result := Open(index);
      vt_High:  result := High(index);
      vt_Low:   result := Low(index);
      vt_Close: result := Close(index);
      else      result := Volume(index);
    end;
  end;

begin
  result := value(StartIndex);
  for i:=1 to count - 1 do
    if value(StartIndex + i) > result then
      result := value(StartIndex + i);
end;

{-----Get lowest value-------------------------------------------------------}
function  GetLowestValue(ValueType: TValueType; StartIndex, count: integer): double;
var
  i: integer;

  function value(index: integer): double;
  begin
    case ValueType of
      vt_Open:  result := Open(index);
      vt_High:  result := High(index);
      vt_Low:   result := Low(index);
      vt_Close: result := Close(index);
      else      result := Volume(index);
    end;
  end;

begin
  result := value(StartIndex);
  for i:=1 to count - 1 do
    if value(StartIndex + i) < result then
      result := value(StartIndex + i);
end;


end.
